'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.store = store;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lowdb = require('lowdb');

var _lowdb2 = _interopRequireDefault(_lowdb);

var _underscoreDb = require('underscore-db');

var _underscoreDb2 = _interopRequireDefault(_underscoreDb);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _schemaScanner = require('./schema/schema-scanner');

var _schemaContainer = require('./schema/schema-container');

var _generator = require('./generator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const STORE_PATH = 'db.json';
const DEFAULT_COUNT = 10;

/**
 * 初始化emock
 * 1. 扫描实体schema、api schema，并dref
 * 2. 随机生成实体数据
 * 3. 根据实体schema中的扩展字段建立关联关系
 * 4.
 *
 *
 * @method store
 * @param {Object} options emock配置
 * @param {string} options.baseFolder schema所在目录
 */
function store(_ref) {
    var _ref$source = _ref.source;
    let source = _ref$source === undefined ? process.cwd() + STORE_PATH : _ref$source;
    let schemaDir = _ref.schemaDir;
    var _ref$count = _ref.count;
    let count = _ref$count === undefined ? DEFAULT_COUNT : _ref$count;

    schemaDir = _path2.default.isAbsolute(schemaDir) ? schemaDir : _path2.default.join(process.cwd(), schemaDir);
    let scanner = new _schemaScanner.SchemaScanner();
    // 从指定目录中扫描schema
    let schemas = scanner.scan(schemaDir);
    // 注册schema到容器中
    schemas.forEach(schema => _schemaContainer.container.registerEntity(schema.id, schema));

    // 如果已经有数据，就不再生成了，如果没有就生成
    let hasData = _fs2.default.existsSync(source);

    if (!_lodash2.default.isString(source)) {
        throw new Error('store文件路径必须是字符串');
    }

    let db = (0, _lowdb2.default)(source);
    db._.mixin(_underscoreDb2.default);

    if (!hasData) {
        initData(db, _schemaContainer.container, _generator.generator, count);
    }

    return db;
}

function initData(store, contianer, generator, count) {
    for (let _ref2 of contianer.getEntities().entries()) {
        var _ref3 = _slicedToArray(_ref2, 2);

        let id = _ref3[0];
        let schema = _ref3[1];

        let collectionId = _pluralize2.default.plural(id);
        store.set(collectionId, []).value();
        let instances = generator.create(schema.schema, count);
        instances.forEach(instance => store.get(collectionId).insert(instance).value());
    }
}